searchNodes=[{"ref":"Crudry.Context.html","title":"Crudry.Context","type":"module","doc":"Generates CRUD functions to DRY Phoenix Contexts. Assumes Ecto.Repo is being used as the repository. Uses the Ecto Schema source name to generate the pluralized name for the functions, and the module name to generate the singular name. This follows the same pattern as the Mix.Tasks.Phx.Gen.Context, so it should be straightforward to replace Phoenix&#39;s auto-generated functions with Crudry. Usage To generate CRUD functions for a given schema, simply do defmodule MyApp.MyContext do alias MyApp.Repo alias MyApp.MySchema require Crudry.Context Crudry.Context.generate_functions MySchema end And the context will have all these functions available: defmodule MyApp.MyContext do alias MyApp.Repo alias MyApp.MySchema require Crudry.Context ## Get functions def get_my_schema(id) do Repo.get(MySchema, id) end def get_my_schema!(id) do Repo.get!(MySchema, id) end def get_my_schema_with_assocs(id, assocs) do MySchema |&gt; Repo.get(id) |&gt; Repo.preload(assocs) end def get_my_schema_with_assocs!(id, assocs) do MySchema |&gt; Repo.get!(id) |&gt; Repo.preload(assocs) end def get_my_schema_by(clauses) do Repo.get_by(MySchema, clauses) end def get_my_schema_by!(clauses) do Repo.get_by!(MySchema, clauses) end def get_my_schema_by_with_assocs(clauses, assocs) do MySchema |&gt; Repo.get_by(clauses) |&gt; Repo.preload(assocs) end def get_my_schema_by_with_assocs!(clauses, assocs) do MySchema |&gt; Repo.get_by!(clauses) |&gt; Repo.preload(assocs) end ## List functions def list_my_schemas() do Repo.all(MySchema) end def list_my_schemas(opts) do MySchema |&gt; Crudry.Query.list(opts) |&gt; Repo.all() end def list_my_schemas_with_assocs(assocs) do MySchema |&gt; Repo.all() |&gt; Repo.preload(assocs) end def list_my_schemas_with_assocs(assocs, opts) do MySchema |&gt; Crudry.Query.list(opts) |&gt; Repo.all() |&gt; Repo.preload(assocs) end def filter_my_schemas(filters) do MySchema |&gt; Crudry.Query.filter(filters) |&gt; Repo.all() end def search_my_schemas(search_term) do module_fields = MySchema.__schema__(:fields) MySchema |&gt; Crudry.Query.search(search_term, module_fields) |&gt; Repo.all() end def count_my_schemas(field \\\\ :id) do Repo.aggregate(MySchema, :count, field) end ## Create functions def create_my_schema(attrs) do %MySchema{} |&gt; MySchema.changeset(attrs) |&gt; Repo.insert() end def create_my_schema!(attrs) do %MySchema{} |&gt; MySchema.changeset(attrs) |&gt; Repo.insert!() end ## Update functions def update_my_schema(%MySchema{} = my_schema, attrs) do my_schema |&gt; MySchema.changeset(attrs) |&gt; Repo.update() end def update_my_schema!(%MySchema{} = my_schema, attrs) do my_schema |&gt; MySchema.changeset(attrs) |&gt; Repo.update!() end def update_my_schema_with_assocs(%MySchema{} = my_schema, attrs, assocs) do my_schema |&gt; Repo.preload(assocs) |&gt; MySchema.changeset(attrs) |&gt; Repo.update() end def update_my_schema_with_assocs!(%MySchema{} = my_schema, attrs, assocs) do my_schema |&gt; Repo.preload(assocs) |&gt; MySchema.changeset(attrs) |&gt; Repo.update!() end ## Delete functions def delete_my_schema(%MySchema{} = my_schema) do my_schema |&gt; Ecto.Changeset.change() |&gt; check_assocs([]) |&gt; Repo.delete() end def delete_my_schema!(%MySchema{} = my_schema) do my_schema |&gt; Ecto.Changeset.change() |&gt; check_assocs([]) |&gt; Repo.delete!() end # Function to check no_assoc_constraints, always generated. defp check_assocs(changeset, nil), do: changeset defp check_assocs(changeset, constraints) do Enum.reduce(constraints, changeset, fn i, acc -&gt; Ecto.Changeset.no_assoc_constraint(acc, i) end) end end"},{"ref":"Crudry.Context.html#default/1","title":"Crudry.Context.default/1","type":"macro","doc":"Sets default options for the context. Options :create - the name of the changeset function used in the create function. Defaults to :changeset. :update - the name of the changeset function used in the update function. Defaults to :changeset. :only - list of functions to be generated. If not empty, functions not specified in this list are not generated. Defaults to []. :except - list of functions to not be generated. If not empty, only functions not specified in this list will be generated. Defaults to []. The accepted values for :only and :except are: [:get, :list, :count, :search, :filter, :create, :update, :delete]. Examples iex&gt; Crudry.Context.default create: :create_changeset, update: :update_changeset :ok iex&gt; Crudry.Context.default only: [:create, :list] :ok iex&gt; Crudry.Context.default except: [:get!, :list, :delete] :ok"},{"ref":"Crudry.Context.html#generate_functions/2","title":"Crudry.Context.generate_functions/2","type":"macro","doc":"Generates CRUD functions for the schema_module. Custom options can be given. To see the available options, refer to the documenation of Crudry.Context.default/1. There is also one extra option that cannot be set by default: check_constraints_on_delete - list of associations that must be empty to allow deletion. Ecto.Changeset.no_assoc_constraint will be called for each association before deleting. Defaults to []. Examples Suppose we want to implement basic CRUD functionality for a User schema, exposed through an Accounts context: defmodule MyApp.Accounts do alias MyApp.Repo require Crudry.Context # Assuming Accounts.User implements a `changeset/2` function, used both to create and update a user. Crudry.Context.generate_functions Accounts.User end Now, suppose the changeset for create and update are different, and we want to delete the record only if the association has_many :assocs is empty: defmodule MyApp.MyContext do alias MyApp.Repo alias MyApp.MySchema require Crudry.Context Crudry.Context.generate_functions MySchema, create: :create_changeset, update: :update_changeset, check_constraints_on_delete: [:assocs] end"},{"ref":"Crudry.Middlewares.TranslateErrors.html","title":"Crudry.Middlewares.TranslateErrors","type":"module","doc":"Absinthe Middleware to translate errors and changeset errors into human readable messages. It support nested changeset errors and internationalization, using Gettext. Usage Cudry.Translator is used by default to translate error messages to the default locale en. You can also use your own Gettext module by adding it to your Absinthe&#39;s schema context/1 function: def context(context) do Map.put(context, :translator, MyAppWeb.Gettext) end Or just override the default locale in your Context Plug: def call(conn, _) do Absinthe.Plug.put_options(conn, context: %{locale: &quot;pt_BR&quot;}) end Then, to handle errors for a field, add it after the resolve, using middleware/2: alias Crudry.Middlewares.TranslateErrors field :create_user, :user do arg :params, non_null(:user_params) resolve &amp;UsersResolver.create_user/2 middleware TranslateErrors end To handle errors for all fields, use middleware/3: alias Crudry.Middlewares.TranslateErrors def middleware(middleware, _field, _object) do middleware ++ [TranslateErrors] end Examples For a simple changeset error: #Ecto.Changeset&lt; action: nil, changes: %{}, errors: [username: {&quot;can&#39;t be blank&quot;, [validation: :required]}], data: #Crudry.User&lt;&gt;, valid?: false &gt; The resulting error will be [&quot;username can&#39;t be blank&quot;] For a changeset with nested errors: #Ecto.Changeset&lt; action: nil, changes: %{ posts: [ #Ecto.Changeset&lt; action: :insert, changes: %{}, errors: [ title: {&quot;can&#39;t be blank&quot;, [validation: :required]}, user_id: {&quot;can&#39;t be blank&quot;, [validation: :required]} ], data: #Crudry.Post&lt;&gt;, valid?: false &gt; ] }, errors: [ username: {&quot;should be at least %{count} character(s)&quot;, [count: 2, validation: :length, kind: :min]} ], data: #Crudry.User&lt;&gt;, valid?: false &gt; The resulting error will be [&quot;posts: title can&#39;t be blank&quot;, &quot;posts: user_id can&#39;t be blank&quot;, &quot;username should be at least 2 character(s)&quot;]"},{"ref":"Crudry.Middlewares.TranslateErrors.html#call/2","title":"Crudry.Middlewares.TranslateErrors.call/2","type":"function","doc":"This is the main middleware callback. It receives an %Absinthe.Resolution{} struct and it needs to return an %Absinthe.Resolution{} struct. The second argument will be whatever value was passed to the middleware call that setup the middleware. Callback implementation for Absinthe.Middleware.call/2."},{"ref":"Crudry.Middlewares.TranslateErrors.html#handle_error/3","title":"Crudry.Middlewares.TranslateErrors.handle_error/3","type":"function","doc":""},{"ref":"Crudry.Query.html","title":"Crudry.Query","type":"module","doc":"Generates Ecto Queries. All functions in this module return an Ecto.Query. Combining the functions in this module can be very powerful. For example, to do pagination with filter and search: pagination_params = %{limit: 10, offset: 1, order_by: &quot;id&quot;, sorting_order: :desc} filter_params = %{username: [&quot;username1&quot;, &quot;username2&quot;]} search_params = %{text: &quot;search text&quot;, fields: [:username]} User |&gt; Crudry.Query.filter(filter_params) |&gt; Crudry.Query.list(pagination_params) |&gt; Crudry.Query.search(search_params.text, search_params.fields) |&gt; Repo.all()"},{"ref":"Crudry.Query.html#filter/2","title":"Crudry.Query.filter/2","type":"function","doc":"Filters the query. Examples Crudry.Query.filter(MySchema, %{id: 5, name: &quot;John&quot;}) Crudry.Query.filter(MySchema, %{name: [&quot;John&quot;, &quot;Doe&quot;]})"},{"ref":"Crudry.Query.html#list/2","title":"Crudry.Query.list/2","type":"function","doc":"Applies some restrictions to the query. Expects opts to be a keyword list or a map containing some of these fields: limit: defaults to not limiting offset: defaults to 0 sorting_order: defaults to :asc (only works if there is also a order_by specified) order_by: defaults to not ordering custom_query: A function that receives the initial query as argument and returns a custom query. Defaults to initial_query Examples Crudry.Query.list(MySchema, [limit: 10]) Crudry.Query.list(MySchema, [limit: 10, offset: 3, sorting_order: :desc, order_by: :value]) Crudry.Query.list(MySchema, %{order_by: &quot;value&quot;}) Crudry.Query.list(MySchema, %{order_by: :value}) Crudry.Query.list(MySchema, custom_query: &amp;MySchema.scope_list/1)"},{"ref":"Crudry.Query.html#search/3","title":"Crudry.Query.search/3","type":"function","doc":"Searches for the search_term in the given fields. Examples Crudry.Query.search(MySchema, &quot;John&quot;, [:name])"},{"ref":"Crudry.Resolver.html","title":"Crudry.Resolver","type":"module","doc":"Generates CRUD functions to DRY Absinthe Resolvers. Requires a previously generated Context. Usage To generate CRUD functions for a given schema resolver, simply do defmodule MyApp.MyResolver do alias MyApp.Repo alias MyApp.MyContext alias MyApp.MySchema require Crudry.Resolver Crudry.Resolver.generate_functions MyContext, MySchema end And the resolver will have all these functions available: defmodule MyApp.MyResolver do alias MyApp.Repo alias MyApp.MyContext alias MyApp.MySchema require Crudry.Resolver def get_my_schema(%{id: id}, _info) do id |&gt; MyContext.get_my_schema() |&gt; nil_to_error(&quot;my_schema&quot;, fn record -&gt; {:ok, record} end) end def list_my_schemas(_args, _info) do {:ok, MyContext.list_my_schemas([])} end def create_my_schema(%{params: params}, _info) do MyContext.create_my_schema(params) end def update_my_schema(%{id: id, params: params}, _info) do id |&gt; MyContext.get_my_schema() |&gt; nil_to_error(&quot;my_schema&quot;, fn record -&gt; MyContext.update_my_schema(record, params) end) end def delete_my_schema(%{id: id}, _info) do id |&gt; MyContext.get_my_schema() |&gt; nil_to_error(&quot;my_schema&quot;, fn record -&gt; MyContext.delete_my_schema(record) end) end # If `result` is `nil`, return an error. Otherwise, apply `func` to the `result`. def nil_to_error(result, name, func) do case result do nil -&gt; {:error, %{message: &quot;not found&quot;, schema: name}} %{} = record -&gt; func.(record) end end def add_info_to_custom_query(custom_query, info) do fn initial_query -&gt; custom_query.(initial_query, info) end end end"},{"ref":"Crudry.Resolver.html#default/1","title":"Crudry.Resolver.default/1","type":"macro","doc":"Sets default options for the resolver. Options :only - list of functions to be generated. If not empty, functions not specified in this list are not generated. Defaults to []. :except - list of functions to not be generated. If not empty, only functions not specified in this list will be generated. Defaults to []. list_opts - options for the list function. See available options in Crudry.Query.list/2. Defaults to []. create_resolver - custom create resolver function with arity 4. Receives the following arguments: [Context, schema_name, args, info]. Defaults to nil. not_found_message - custom message for the nil_to_error function. Defaults to &quot;not found&quot;. primary_key - custom primary key argument to use in get, update and delete resolvers. Defaults to :id. Note: in list_opts, custom_query will receive absinthe&#39;s info as the second argument and, therefore, must have arity 2. See example in generate_functions/3. The accepted values for :only and :except are: [:get, :list, :create, :update, :delete]. Examples iex&gt; Crudry.Resolver.default only: [:create, :list] :ok iex&gt; Crudry.Resolver.default except: [:get!, :list, :delete] :ok iex&gt; Crudry.Resolver.default list_opts: [order_by: :id] :ok iex&gt; Crudry.Resolver.default list_opts: [custom_query: &amp;scope_list/2] :ok"},{"ref":"Crudry.Resolver.html#generate_functions/3","title":"Crudry.Resolver.generate_functions/3","type":"macro","doc":"Generates CRUD functions for the schema_module resolver. Custom options can be given. To see the available options, refer to the documenation of Crudry.Resolver.default/1, noting that the not_found_message must only be configured using default/1. Examples Overriding functions To define a custom function, add it to except and define your own. In the following example, the schema is updated with its association. defmodule MyApp.Resolver do alias MyApp.Repo alias MyApp.MyContext alias MyApp.MySchema require Crudry.Resolver Crudry.Resolver.generate_functions MyContext, MySchema, except: [:update] def update_my_schema(%{id: id, params: params}, _info) do id |&gt; MyContext.get_my_schema() |&gt; nil_to_error(&quot;My Schema&quot;, fn record -&gt; MyContext.update_my_schema_with_assocs(record, params, [:assoc]) end) end end By using the nil_to_error function, we DRY the nil checking and also ensure the error message is the same as the other auto-generated functions. Custom create resolver It&#39;s possible to define a custom create function, useful when all functions in a resolver change the data in the same way before creating: defmodule MyApp.Resolver do alias MyApp.Repo alias MyApp.MyContext alias MyApp.MySchema require Crudry.Resolver Crudry.Resolver.default create_resolver: &amp;create_resolver/4) Crudry.Resolver.generate_functions MyContext, MySchema Crudry.Resolver.generate_functions MyContext, OtherSchema def create_resolver(context, schema_name, args, %{context: %{current_user: %{company_id: company_id}}}) do apply(context, :&quot;create_\#{schema_name}&quot;, [Map.put(args.params, :company_id, company_id)]) end end Now when creating MySchema and OtherSchema, the custom function will put the current user&#39;s company_id in the params. Custom query It&#39;s also possible to use a custom query that has access to absinthe&#39;s info: defmodule MyApp.Resolver do alias MyApp.Repo alias MyApp.MyContext alias MyApp.MySchema require Crudry.Resolver def scope_list(MySchema, %{context: %{current_user: current_user}} = _info) do where(MySchema, [p], p.user_id == ^current_user.id) end Crudry.Resolver.generate_functions MyContext, MySchema, list_opts: [custom_query: &amp;scope_list/2] end With this, the list function will effectively become: def list_my_schemas(_args, info) do {:ok, MyContext.list_my_schemas(custom_query: add_info_to_custom_query(scope_list, info))} end See Crudry.Query.list/2 for further information on the custom_query option."},{"ref":"Crudry.Translator.html","title":"Crudry.Translator","type":"module","doc":"A module providing Internationalization with a gettext-based API. By using Gettext, your module gains a set of macros for translations, for example: import Crudry.Gettext # Simple translation gettext(&quot;Here is the string to translate&quot;) # Plural translation ngettext(&quot;Here is the string to translate&quot;, &quot;Here are the strings to translate&quot;, 3) # Domain-based translation dgettext(&quot;errors&quot;, &quot;Here is the error message to translate&quot;) See the Gettext Docs for detailed usage. Usage with Crudry.Middlewares.TranslateErrors This module defines errors_domain/0 and schemas_domain/0 functions, which return the domains that will be used to translate changeset errors and ecto schema keys. You can also define and use your own Translator module in Crudry.Middlewares.TranslateErrors by adding it to your Absinthe&#39;s schema context/1 function: def context(context) do Map.put(context, :translator, MyApp.Translator) end"},{"ref":"Crudry.Translator.html#dgettext/3","title":"Crudry.Translator.dgettext/3","type":"macro","doc":""},{"ref":"Crudry.Translator.html#dgettext_noop/2","title":"Crudry.Translator.dgettext_noop/2","type":"macro","doc":""},{"ref":"Crudry.Translator.html#dngettext/5","title":"Crudry.Translator.dngettext/5","type":"macro","doc":""},{"ref":"Crudry.Translator.html#dngettext_noop/3","title":"Crudry.Translator.dngettext_noop/3","type":"macro","doc":""},{"ref":"Crudry.Translator.html#errors_domain/0","title":"Crudry.Translator.errors_domain/0","type":"function","doc":""},{"ref":"Crudry.Translator.html#gettext/2","title":"Crudry.Translator.gettext/2","type":"macro","doc":""},{"ref":"Crudry.Translator.html#gettext_comment/1","title":"Crudry.Translator.gettext_comment/1","type":"macro","doc":""},{"ref":"Crudry.Translator.html#gettext_noop/1","title":"Crudry.Translator.gettext_noop/1","type":"macro","doc":""},{"ref":"Crudry.Translator.html#handle_missing_bindings/2","title":"Crudry.Translator.handle_missing_bindings/2","type":"function","doc":"Default handling for missing bindings. This function is called when there are missing bindings in a translation. It takes a Gettext.MissingBindingsError struct and the translation with the wrong bindings left as is with the %{} syntax. For example, if something like this is called: MyApp.Gettext.gettext(&quot;Hello %{name}, welcome to %{country}&quot;, name: &quot;Jane&quot;, country: &quot;Italy&quot;) and our it/LC_MESSAGES/default.po looks like this: msgid &quot;Hello %{name}, welcome to %{country}&quot; msgstr &quot;Ciao %{name}, benvenuto in %{cowntry}&quot; # (typo) then Gettext will call: MyApp.Gettext.handle_missing_bindings(exception, &quot;Ciao Jane, benvenuto in %{cowntry}&quot;) where exception is a struct that looks like this: %Gettext.MissingBindingsError{ backend: MyApp.Gettext, domain: &quot;default&quot;, locale: &quot;it&quot;, msgid: &quot;Hello %{name}, welcome to %{country}&quot;, bindings: [:country], } The return value of the c:handle_missing_bindings/2 callback is used as the translated string that the translation macros and functions return. The default implementation for this function uses Logger.error/1 to warn about the missing binding and returns the translated message with the incomplete bindings. This function can be overridden. For example, to raise when there are missing bindings: def handle_missing_bindings(exception, _incomplete) do raise exception end Callback implementation for Gettext.Backend.handle_missing_bindings/2."},{"ref":"Crudry.Translator.html#handle_missing_plural_translation/6","title":"Crudry.Translator.handle_missing_plural_translation/6","type":"function","doc":"Default handling for plural translations with a missing translation. Same as c:handle_missing_translation/4, but for plural translations. In this case, n is the number used for pluralizing the translated string. Callback implementation for Gettext.Backend.handle_missing_plural_translation/6."},{"ref":"Crudry.Translator.html#handle_missing_translation/4","title":"Crudry.Translator.handle_missing_translation/4","type":"function","doc":"Default handling for translations with a missing translation. When a Gettext function/macro is called with a string to translate into a locale but that locale doesn&#39;t provide a translation for that string, this callback is invoked. msgid is the string that Gettext tried to translate. This function should return {:ok, translated} if a translation can be fetched or constructed for the given string, or {:default, msgid} otherwise. Callback implementation for Gettext.Backend.handle_missing_translation/4."},{"ref":"Crudry.Translator.html#lgettext/4","title":"Crudry.Translator.lgettext/4","type":"function","doc":""},{"ref":"Crudry.Translator.html#lngettext/6","title":"Crudry.Translator.lngettext/6","type":"function","doc":""},{"ref":"Crudry.Translator.html#ngettext/4","title":"Crudry.Translator.ngettext/4","type":"macro","doc":""},{"ref":"Crudry.Translator.html#ngettext_noop/2","title":"Crudry.Translator.ngettext_noop/2","type":"macro","doc":""},{"ref":"Crudry.Translator.html#schemas_domain/0","title":"Crudry.Translator.schemas_domain/0","type":"function","doc":""}]